<?php

/**
 * @file
 * Contains \Drupal\drd\Crypt\Method\Openssl.
 */

namespace Drupal\drd\Crypt\Method;

use Drupal\Core\Form\FormStateInterface;
use Drupal\drd\Crypt\BaseMethod;

/**
 * Provides OpenSSL encryption functionality.
 *
 * @ingroup drd
 */
class Openssl extends BaseMethod {

  private $cipher;

  private $iv;

  private $password;

  public function __construct($settings = array()) {
    $this->cipher = isset($settings['cipher']) ? $settings['cipher'] : 'aes-128-cbc';
    $this->password = !empty($settings['password']) ? $settings['password'] : '';
  }

  /**
   * @inheritDoc
   */
  public function getLabel() {
    return 'OpenSSL';
  }

  /**
   * @inheritDoc
   */
  public function isAvailable() {
    return function_exists('openssl_encrypt');
  }

  /**
   * @inheritDoc
   */
  public function getCipherMethods() {
    return openssl_get_cipher_methods();
  }

  /**
   * @inheritDoc
   */
  public function settingsForm(array &$form, array $condition) {
    $form['openssl_cipher'] = array(
      '#type' => 'select',
      '#title' => t('Cipher'),
      '#options' => $this->getCipherMethods(),
      '#default_value' => array_search($this->cipher, $this->getCipherMethods()),
      '#states' => array(
        'required' => $condition,
      ),
    );
    $form['openssl_password'] = array(
      '#type' => 'textfield',
      '#title' => t('Password / Key'),
      '#default_value' => $this->password,
      '#states' => array(
        'required' => $condition,
      ),
    );
  }

  /**
   * @inheritDoc
   */
  public function settingsFormValues(FormStateInterface $form_state) {
    $ciphers = $this->getCipherMethods();
    return array(
      'cipher' => $ciphers[$form_state->getValue('openssl_cipher')],
      'password' => $form_state->getValue('openssl_password'),
    );
  }

  /**
   * @inheritDoc
   */
  public function getIV() {
    if (empty($this->iv)) {
      $nonceSize = openssl_cipher_iv_length($this->cipher);
      $this->iv = openssl_random_pseudo_bytes($nonceSize);
    }
    return $this->iv;
  }

  /**
   * @inheritDoc
   */
  public function encrypt($args) {
    return openssl_encrypt(
      serialize($args),
      $this->cipher,
      $this->password,
      OPENSSL_RAW_DATA,
      $this->getIV()
    );
  }

  /**
   * @inheritDoc
   */
  public function decrypt($body, $iv) {
    $this->iv = $iv;
    return unserialize(openssl_decrypt(
      $body,
      $this->cipher,
      $this->password,
      OPENSSL_RAW_DATA,
      $this->iv
    ));
  }

}

if (!defined('OPENSSL_RAW_DATA')) {
  define('OPENSSL_RAW_DATA', 1);
}
